#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define NLINE 20
#define NCOL 36

#define START_LINE 0
#define START_COL 0
#define GOAL_LINE (NLINE-1)
#define GOAL_COL (NCOL-1)

static char
wallright[NCOL * NLINE],
wallbelow[NCOL * NLINE],
reachable[NCOL * NLINE];

static void
initmap(void) {
	int pos;
	for (pos = 0; pos < NLINE * NCOL; pos++) {
		wallright[pos] = 1;
		wallbelow[pos] = 1;
		reachable[pos] = 0;
	}
	reachable[NCOL * START_LINE + START_COL] = 1;
}

#define REACH_QUEUE_SIZE (4 * NCOL * NLINE)
static void
reach(int line, int col) {
	int linequeue[REACH_QUEUE_SIZE], colqueue[REACH_QUEUE_SIZE];
	int head, tail;
	linequeue[0] = line;
	colqueue[0] = col;
	head = 0;
	tail = 1;
	while (head < tail) {
		int l, c;
		l = linequeue[head];
		c = colqueue[head++];
		if (!reachable[NCOL * l + c]) {
			reachable[NCOL * l + c] = 1;
			if (0 <= l - 1 && !wallbelow[NCOL * (l - 1) + c]) {
				linequeue[tail] = l - 1; 
				colqueue[tail++] = c;
			}
			if (0 <= c - 1 && !wallright[NCOL * l + (c - 1)]) {
				linequeue[tail] = l; 
				colqueue[tail++] = c - 1;
			}
			if (c + 1 < NCOL && !wallright[NCOL * l + c]) {
				linequeue[tail] = l;
				colqueue[tail++] = c + 1;
			}
			if (l + 1 < NLINE && !wallbelow[NCOL * l + c]) {
				linequeue[tail] = l + 1;
				colqueue[tail++] = c;
			}
		}
	}
}

static void
digright(int line, int col) {
	wallright[NCOL * line + col] = 0;
	if (reachable[NCOL * line + col])
		reach(line, col + 1);
	if (reachable[NCOL * line + (col + 1)])
		reach(line, col);
}

static void
digbelow(int line, int col) {
	wallbelow[NCOL * line + col] = 0;
	if (reachable[NCOL * line + col])
		reach(line + 1, col);
	if (reachable[NCOL * (line + 1) + col])
		reach(line, col);
}

static int 
randint(int limit) {
	return (int)(limit * ((double)rand() / ((double)RAND_MAX + 1)));
}

static void
dig_rand(void) {
	int line, col;
	line = randint(NLINE);
	col = randint(NCOL);
	if (randint(2)) {
		if (col < NCOL - 1)
			digright(line, col);
	} else {
		if (line < NLINE - 1)
			digbelow(line, col);
	}
}

static int
goal_reachable(void) {
	return reachable[NCOL * GOAL_LINE + GOAL_COL];
}

static void
printmap(void) {
	int line, col;
	putchar('+');
	for (col = 0; col < NCOL; col++) {
		putchar('-');
		putchar('+');
	}
	putchar('\n');
	for (line = 0; line < NLINE; line++) {
		putchar('|');
		for (col = 0; col < NCOL; col++) {
			if (START_LINE == line && START_COL == col)
				putchar('@');
			else if (GOAL_LINE == line && GOAL_COL == col)
				putchar('>');
			else
				putchar(' ');
			if (wallright[NCOL * line + col])
				putchar('|');
			else
				putchar(' ');
		}
		putchar('\n');
		putchar('+');
		for (col = 0; col < NCOL; col++) {
			if (wallbelow[NCOL * line + col])
				putchar('-');
			else
				putchar(' ');
			putchar('+');
		}
		putchar('\n');
	}
}

int 
main(void) {
	srand(time(0));
	initmap();
	while (!goal_reachable()) {
		dig_rand();
	}
	printmap();
	return 0;
}

